-- (c) M.P.I 98/99 & 99/00 -- Last Update: 99.11.22

module RList where

import Mpi

-- (1) Datatype definition -----------------------------------------------------

data RList a = Nil | Cons (a , RList a) deriving Show

inRList = either (const Nil) Cons

outRList Nil         = Left ()
outRList (Cons(a,x)) = Right(a,x)

-- (2) Ana + cata + hylo -------------------------------------------------------

cataRList g = g . recRList (cataRList g) . outRList   

anaRList g = inRList . (recRList (anaRList g) ) . g

hyloRList h g = cataRList h . anaRList g

recRList f = id -|- (id >< f)

-- (3) Map ---------------------------------------------------------------------

instance Functor RList
         where fmap f = cataRList ( inRList . (id -|- (f >< id)))

-- (4) Examples ----------------------------------------------------------------

-- (4.1) Factorial--------------------------------------------------------------

fac = hyloRList (either (const 1) mul) obsNat

-- where

mul(x,y) = x*y

obsNat 0 = Left ()
obsNat n = Right (n,n-1)

-- (4.2) Insertion sort --------------------------------------------------------

iSort :: Ord a => [a] -> [a]
iSort = hyloRList (either (const []) insert) conv

-- where

conv []    = Left  ()
conv (h:t) = Right (h,t)

insert(x,[])              = [x]
insert(x,a:l) | x < a     = [x,a]++l
              | otherwise = a:(insert(x,l))

-- (4.3) converting RList to haskell lists -------------------------------------

cRList2h = cataRList (either (const []) (uncurry (:)))

ch2RList = foldr (curry Cons) Nil

--------------------------------------------------------------------------------

